==> gateau-0.1.5.tgz: package/es/state/index.js <==
import R from 'ramda';
import { extendObservable, observable, action } from 'mobx';

import { parseInput } from '../parser';
import validateContent, { transformerForType, stringRepresenterForType } from '../ingredients/validateContent';

var suggestReferenceFromTree = R.uncurryN(2, function (ingredients) {
	return R.pipe(R.chain(R.pluck('references')), R.unnest, R.map(R.head), // Just the requested ingredient ID
	R.difference(R.__, R.pluck('id', ingredients)), // Only IDs that are yet to be used
	R.head // First pick
	);
});

function createObservableIngredientVariation(ingredient, _ref) {
	var _ref$enabled = _ref.enabled,
	    enabled = _ref$enabled === undefined ? true : _ref$enabled,
	    rawContent = _ref.rawContent;

	return observable({
		enabled: enabled,
		rawContent: rawContent,
		get result() {
			var transform = transformerForType(ingredient.type);
			return R.tryCatch(transform, R.objOf('error'))(this.rawContent);
		},
		toggleEnabled: action(function () {
			this.enabled = !this.enabled;
		}),
		adjustRawContent: action(function (adjuster) {
			this.rawContent = adjuster(this.rawContent);
		}),
		adjustContent: action(function (adjuster) {
			var result = this.result;
			if (result.content) {
				adjuster(result.content);
				this.rawContent = stringRepresenterForType(ingredient.type, result.content);
			}
		}),
		editPath: action(function (path, editor) {
			this.adjustContent(function (content) {
				var _ref2 = path.length == 1 ? [content, path[0]] : [R.path(initialPath, content), R.last(path)],
				    parent = _ref2[0],
				    key = _ref2[1];

				if (parent) {
					editor(parent, key);
				} else {
					// Warn user about incorrect key path?
				}
			});
		}),
		adjustPath: action(function (path, adjuster) {
			this.editPath(path, function (parent, key) {
				parent[key] = adjuster(parent[key]);
			});
		})
	});
}

function createObservableIngredient(target) {
	return extendObservable(observable(target), {
		variations: target.variations.map(R.curry(createObservableIngredientVariation)(target)),
		addVariation: action(function (variation) {
			target.variations.push(createObservableIngredientVariation(target, variation));
		}),
		addNewVariation: action(function (variation) {
			target.addVariation({
				rawContent: ''
			});
		}),
		get flattenedResult() {
			var flattened = {
				type: target.type,
				variationReference: R.last(target.variations)
			};

			if (target.type == 'json') {
				flattened.content = target.variations.reduce(function (combined, _ref3) {
					var enabled = _ref3.enabled,
					    result = _ref3.result;

					if (!enabled || result.content == null) {
						return combined;
					}

					return Object.assign(combined, result.content);
				}, {});
			} else {
				var variation = R.findLast(R.propEq('enabled', true), target.variations);
				if (variation) {
					flattened.content = variation.result.content;
				}
			}

			return flattened;
		}
	});
}

export default function createObservableState(_ref4) {
	var content = _ref4.content,
	    allIngredients = _ref4.allIngredients,
	    scenarios = _ref4.scenarios,
	    activeScenarioIndex = _ref4.activeScenarioIndex,
	    destinationID = _ref4.destinationID,
	    destinationDevice = _ref4.destinationDevice;

	var target = this;
	return extendObservable(target, {
		content: content,
		get contentTree() {
			return parseInput(this.content);
		},
		allIngredients: allIngredients.map(createObservableIngredient),
		scenarios: scenarios,
		activeScenarioIndex: activeScenarioIndex,
		destinationID: destinationID,
		destinationDevice: destinationDevice,
		get activeScenario() {
			return this.scenarios[this.activeScenarioIndex];
		},
		get activeIngredients() {
			var activeScenario = this.activeScenario;
			return this.allIngredients.reduce(function (object, _ref5) {
				var id = _ref5.id,
				    flattenedResult = _ref5.flattenedResult;

				object[id] = flattenedResult;
				return object;
			}, {});
		},
		activeVariationForIngredientAtIndex: function activeVariationForIngredientAtIndex(index) {
			var ingredient = this.allIngredients[index];
			if (ingredient && ingredient.variations.length > 0) {
				var activeScenario = this.activeScenario;
				return ingredient.variations[R.propOr(0, ingredient.id, activeScenario)];
			}
		},

		addNewIngredient: action(function () {
			target.allIngredients.push(createObservableIngredient({
				id: R.defaultTo('untitled', suggestReferenceFromTree(target.allIngredients, target.contentTree)),
				type: 'text',
				variations: [{
					rawContent: ''
				}]
			}));
		}),
		// Use target to allow prebinding
		onRemoveIngredientAtIndex: action(function (index) {
			target.allIngredients.splice(index, 1);
		})
	});
}

==> gateau-0.1.6.tgz: package/es/state/index.js <==
import R from 'ramda';
import { extendObservable, observable, action } from 'mobx';

import { parseInput } from '../parser';
import validateContent, { transformerForType, stringRepresenterForType } from '../ingredients/validateContent';

var suggestReferenceFromTree = R.uncurryN(2, function (ingredients) {
	return R.pipe(R.chain(R.pluck('references')), R.unnest, R.map(R.head), // Just the requested ingredient ID
	R.difference(R.__, R.pluck('id', ingredients)), // Only IDs that are yet to be used
	R.head // First pick
	);
});

function createObservableIngredientVariation(ingredient, _ref) {
	var _ref$enabled = _ref.enabled,
	    enabled = _ref$enabled === undefined ? true : _ref$enabled,
	    rawContent = _ref.rawContent;

	return observable({
		enabled: enabled,
		rawContent: rawContent,
		get result() {
			var transform = transformerForType(ingredient.type);
			return R.tryCatch(transform, R.objOf('error'))(this.rawContent);
		},
		toggleEnabled: action(function () {
			this.enabled = !this.enabled;
		}),
		adjustRawContent: action(function (adjuster) {
			this.rawContent = adjuster(this.rawContent);
		}),
		adjustContent: action(function (adjuster) {
			var result = this.result;
			if (result.content) {
				adjuster(result.content);
				this.rawContent = stringRepresenterForType(ingredient.type, result.content);
			}
		}),
		editPath: action(function (path, editor) {
			this.adjustContent(function (content) {
				var _ref2 = path.length == 1 ? [content, path[0]] : [R.path(initialPath, content), R.last(path)],
				    parent = _ref2[0],
				    key = _ref2[1];

				if (parent) {
					editor(parent, key);
				} else {
					// Warn user about incorrect key path?
				}
			});
		}),
		adjustPath: action(function (path, adjuster) {
			this.editPath(path, function (parent, key) {
				parent[key] = adjuster(parent[key]);
			});
		})
	});
}

function createObservableIngredient(target) {
	return extendObservable(observable(target), {
		variations: target.variations.map(R.curry(createObservableIngredientVariation)(target)),
		addVariation: action(function (variation) {
			target.variations.push(createObservableIngredientVariation(target, variation));
		}),
		addNewVariation: action(function (variation) {
			target.addVariation({
				rawContent: ''
			});
		}),
		get flattenedResult() {
			var flattened = {
				type: target.type,
				variationReference: R.last(target.variations)
			};

			if (target.type == 'json') {
				flattened.content = target.variations.reduce(function (combined, _ref3) {
					var enabled = _ref3.enabled,
					    result = _ref3.result;

					if (!enabled || result.content == null) {
						return combined;
					}

					return Object.assign(combined, result.content);
				}, {});
			} else {
				var variation = R.findLast(R.propEq('enabled', true), target.variations);
				if (variation) {
					flattened.content = variation.result.content;
				}
			}

			return flattened;
		}
	});
}

export default function createObservableStateManager() {
	var _ref4 = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {},
	    _ref4$content = _ref4.content,
	    content = _ref4$content === undefined ? '' : _ref4$content,
	    _ref4$destinationID = _ref4.destinationID,
	    destinationID = _ref4$destinationID === undefined ? 'foundation' : _ref4$destinationID,
	    _ref4$destinationDevi = _ref4.destinationDevice,
	    destinationDevice = _ref4$destinationDevi === undefined ? 'phone' : _ref4$destinationDevi,
	    _ref4$ingredients = _ref4.ingredients,
	    ingredients = _ref4$ingredients === undefined ? [] : _ref4$ingredients,
	    _ref4$scenarios = _ref4.scenarios,
	    scenarios = _ref4$scenarios === undefined ? [{}] : _ref4$scenarios,
	    _ref4$activeScenarioI = _ref4.activeScenarioIndex,
	    activeScenarioIndex = _ref4$activeScenarioI === undefined ? 0 : _ref4$activeScenarioI;

	return extendObservable({}, {
		content: content,
		destinationID: destinationID,
		destinationDevice: destinationDevice,
		ingredients: ingredients.map(createObservableIngredient),
		scenarios: scenarios,
		activeScenarioIndex: activeScenarioIndex,

		get contentTree() {
			return parseInput(this.content);
		},
		get activeScenario() {
			return this.scenarios[this.activeScenarioIndex];
		},
		get activeIngredients() {
			var activeScenario = this.activeScenario;
			return this.ingredients.reduce(function (object, _ref5) {
				var id = _ref5.id,
				    flattenedResult = _ref5.flattenedResult;

				object[id] = flattenedResult;
				return object;
			}, {});
		},
		activeVariationForIngredientAtIndex: function activeVariationForIngredientAtIndex(index) {
			var ingredient = this.ingredients[index];
			if (ingredient && ingredient.variations.length > 0) {
				var activeScenario = this.activeScenario;
				return ingredient.variations[R.propOr(0, ingredient.id, activeScenario)];
			}
		},

		addNewIngredient: action.bound(function () {
			this.ingredients.push(createObservableIngredient({
				id: R.defaultTo('untitled', suggestReferenceFromTree(this.ingredients, this.contentTree)),
				type: 'text',
				variations: [{
					rawContent: ''
				}]
			}));
		}),
		onRemoveIngredientAtIndex: action.bound(function (index) {
			this.ingredients.splice(index, 1);
		})
	});
}
